/// <reference path="./frame/externals.d.ts" />

namespace Components {

    export interface ITestComponentProps {
        children?: React.ReactChild;
        stateParam?: string;
    }

    export interface ITestComponentState {
        stateParam: string;
    }

    export class TestComponent extends React.Component<ITestComponentProps, ITestComponentState> {

        private mouseDownHandler: any;
        private mouseUpHandler: any;

        constructor(props: ITestComponentProps) {
            super(props);
            this.state = {
                stateParam: this.props.stateParam || ""
            } as ITestComponentState;
        }

        componentDidMount() {
            this.attachEvents();
        }

        componentWillUnmount() {
            this.detachEvents();
        }

        shouldComponentUpdate(prevProps: ITestComponentProps, prevState: ITestComponentState, context: any) {
            return prevState.stateParam !== this.state.stateParam;
        }

        private mouseDownListener(e: MouseEvent) {
        }

        private mouseUpListener(e: MouseEvent) {
        }

        private get target(): HTMLCanvasElement {
            return this.refs["target"] as HTMLCanvasElement;
        }

        private attachEvents() {
            if(!this.target) return;
            this.target.addEventListener("mousedown", this.mouseDownHandler);
            this.target.addEventListener("mouseup", this.mouseUpHandler);
        }

        private detachEvents() {
            if(!this.target) return;
            this.target.removeEventListener("mousedown", this.mouseDownHandler);
            this.target.removeEventListener("mouseup", this.mouseUpHandler);
        }

        render() {
            return (
                <div ref="target" className="target">
                    Hello TestComponent
                </div>
            );
        }
    }
}